#!/util/bin/perl

use CGI;
use Bind;

$doc = new CGI;
$dns = new Bind("/local/stock/Bind/domain/named.boot");

sub error {
    my($message) = shift;

    print $message,"<br>\n";
    exit(0);
}

# This function takes data given by the user and tries to modify the database.
sub changedata {
    my(@rrs,$i,%set);

    print $doc->header;
    print $doc->start_html("Result of Edits to $R::host");

    # If we were told to delete the zone, we do it first.
    if ($R::del_zone) {
	foreach ($dns->listzone($R::zone)) {
	    $dns->delete($_,'ALL');
	    print "Deleted $_<br>\n";
	}
	return;
    }

    # fully-qualify all hostname input.
    $R::name = $dns->fqdn($R::name,$R::zone);
    $R::cname = $dns->fqdn($R::cname,$R::zone);
    $R::soa_pri = $dns->fqdn($R::soa_pri,$R::zone);
    $R::soa_contact = $dns->fqdn($R::soa_contact,$R::zone);
    $i=0;
    while ($doc->param("ns$i")) {
	$doc->param("ns$i",$dns->fqdn($doc->param("ns$i"),$R::zone));
	$i++;
    }
    $i=0;
    while ($doc->param("mx_host$i")) {
	$doc->param("mx_host$i",$dns->fqdn($doc->param("mx_host$i"),$R::zone));
	$i++;
    }

    # Start the tests...

    # Insure that all MX preferences are unique.
    $i=0;
    while ($doc->param("mx_pref$i")) {
	if ($set{$doc->param("mx_pref$i")}) {
	    &error("MX values aren't unique.");
	}
	$set{$doc->param("mx_pref$i")} = 1;
	$i++;
    }
    undef(%set); # Clean up.add
    if (defined($R::soa_pri)) { # Do the SOA tests.
	if ($R::soa_refresh !~ /^[0-9]+$/ ||
	    $R::soa_retry !~ /^[0-9]+$/||
	    $R::soa_expire !~ /^[0-9]+$/||
	    $R::soa_ttl !~ /^[0-9]+$/) {
	    &error("SOA timers must be positive whole numbers.");
	}
	if ($R::soa_refresh < $R::soa_retry) {
	    &error("SOA retry must be less than the refresh time.");
	}
	if ($R::soa_expire < $R::soa_refresh) {
	    &error("SOA refresh must be less than the expire time.");
	}
	if ($R::soa_ttl > $R::soa_expire) {
	    &error("SOA default TTL must be less than the expire time.");
	}
    }

    print "Tests successful.<br>\n";

    # We do the mods in bulk.  We can print out the old and new data in the
    # transaction log and a human who cares can sort out what was changed.
    @rrs = $dns->list($R::host,'ALL');
    $dns->delete($R::host,'ALL');

    if ($R::del_host) {
	print "Host deleted.<br>\n";
	return;
    }

    if ($R::host ne $R::name) {
	print "Hostname changed.<br>\n";
    }

    # Construct some of the new entries.
    if ($R::soa_pri) {
	$newsoa = sprintf("%s %s ( %d %d %d %d %d )",
			  $R::soa_pri,
			  $R::soa_contact,
			  $R::soa_serial,
			  $R::soa_refresh,
			  $R::soa_retry,
			  $R::soa_expire,
			  $R::soa_ttl);
    }
    $i=0;
    while ($doc->param("mx_host$i")) {
	$newmx[$i] = $doc->param("mx_pref$i")." ".$doc->param("mx_host$i");
	$i++;
    }

    # Now we add them all back in.

    if ($R::soa_pri) {
	$dns->add($R::name, $R::zone, $R::newsoa_ttl, 'SOA', $newsoa);
    }
    $i=0;
    while ($doc->param("ns$i")) {
	$dns->add($R::name, $R::zone, $doc->param("ns$i_ttl"), 'NS', 
		  $doc->param("ns$i"));
	$i++;
    }
    $i=0;
    while ($doc->param("a$i")) {
	my($rev) = join('.',(split(/\./,$doc->param("a$i")))[3,2,1,0]).
	    ".IN-ADDR.ARPA.";
	my($revzone) = join('.',(split(/\./,$doc->param("a$i")))[2,1,0]).
	    ".IN-ADDR.ARPA.";
	$dns->delete($rev, 'PTR'); # We really shouldn't do this.
	$dns->add($R::name, $R::zone, $doc->param("a$i_ttl"), 'A', 
		  $doc->param("a$i"));
	$dns->add($rev, $revzone, $doc->param("a$i_ttl"), 'PTR', $R::name);
	$i++;
    }
    $i=0;
    while ($newmx[$i]) {
	$dns->add($R::name, $R::zone, $doc->param("mx$i_ttl"), 'MX', 
		  $newmx[$i]);
	$i++;
    }
    if ($R::hinfo) {
	$dns->add($R::name, $R::zone, $R::hinfo_ttl, 'HINFO', $R::hinfo);
    }
    if ($R::cname) {
	$dns->add($R::name, $R::zone, $R::cname_ttl, 'CNAME', $R::cname);
    }
    if ($R::txt) {
	$dns->add($R::name, $R::zone, $R::txt_ttl, 'TXT', $R::txt);
    }
    print $doc->h1("Old RRs");
    print $doc->hr;
    $host = $R::host;
    print "<table>\n";
    foreach $record (@rrs) {
	($type, $data) = split(/:/,$record);
	print $doc->TR($doc->th($host),
		       $doc->th($doc->strong($type)),
		       $doc->th($data)),"\n";
	$host = "";
    }
    print "</table>\n";

    print $doc->h1("New RRs");
    print $doc->hr;
    $host = $R::name;
    print "<table>\n";
    foreach $record ($dns->list($host,'all')) {
	($type, $data) = split(/:/,$record);
	print $doc->TR($doc->th($host),
		       $doc->th($doc->strong($type)),
		       $doc->th($data)),"\n";
	$host = "";
    }
    print "</table>\n";
}

$doc->import_names('R');

if (!defined($R::host) || !defined($R::zone)) {
    print $doc->start_html("Error");
    print "Script invocation is invalid.\n";
    exit(0);
}

if ($ENV{'REQUEST_METHOD'} eq 'POST') {
    &changedata;
    exit(0);
}

$host = $R::host;

print $doc->header;
print $doc->start_html("Edit $host");

print $doc->startform(-method=>'post');

print "Hostname: ",$doc->textfield(-name=>'name',
				   -default=>$host,
				   -size=>40,
				   -maxlength=>255);
print $doc->checkbox(-name=>"del_host",
		     -label=>'Delete all records');

$mxinc=0;
$ainc=0;
$nsinc=0;
foreach $record ($dns->list($host,'all')) {
    my($type, $data) = split(/:/,$record);

    if ($type ne $oldtype) {
	print "<hr>";
	if ($type eq 'soa') {
	    print $doc->h3("SOA");
	    print $doc->checkbox(-name=>'del_zone',
				 -label=>'Delete zone'),"<br>\n";
	}
	if ($type eq 'ns') {
	    print $doc->h3("Nameservers");
	}	    
	if ($type eq 'mx') {
	    print $doc->h3("Mail Exchangers");
	}	    
	if ($type eq 'a') {
	    print $doc->h3("Addresses");
	}	    
    }
    $oldtype = $type;

    if ($type eq 'soa') {
	@_ = split(/\s+/,$data);

	print "Primary nameserver: ",$doc->textfield(-name=>"soa_pri",
						     -default=>$_[0],
						     -size=>40,
						     -maxlength=>255),"<br>\n";
	print "Contact: ",$doc->textfield(-name=>"soa_contact",
					  -default=>$_[1],
					  -size=>40),"<br>\n";
	print $doc->hidden(-name=>"soa_serial",
			   -default=>$_[3]);
	print "Refresh: ",$doc->textfield(-name=>"soa_refresh",
					  -default=>$_[4],
					  -size=>20),"<br>\n";
	print "Retry: ",$doc->textfield(-name=>"soa_retry",
					-default=>$_[5],
					-size=>20),"<br>\n";
	print "Expire: ",$doc->textfield(-name=>"soa_expire",
					 -default=>$_[6],
					 -size=>20),"<br>\n";
	print "Default TTL: ",$doc->textfield(-name=>"soa_ttl",
					      -default=>$_[7],
					      -size=>20),"<br>\n";
    }
    if ($type eq 'ns') {
	print $doc->textfield(-name=>"ns$nsinc",
			      -default=>$data,
			      -size=>40,
			      -maxlength=>255);
	print $doc->checkbox(-name=>"del_ns$nsinc",
			     -label=>'Delete this record'),"<br>\n";
	$nsinc++;
    }
    if ($type eq 'a') {
	($ttl, $ip) = split(/\s/,$data);
	if (defined($ip)) {
	    print "TTL: ", $doc->textfield(-name=>"a$inc_ttl",
				  -default=>$ttl,
				  -size=>5,
				  -maxlength=>6);
	    print "IP: ",$doc->textfield(-name=>"a$ainc",
				  -default=>$ip,
				  -size=>18,
				  -maxlength=>16);
	} else {
	    print $doc->textfield(-name=>"a$ainc",
				  -default=>$data,
				  -size=>18,
				  -maxlength=>16);
	}
	print $doc->checkbox(-name=>"del_a$ainc",
			     -label=>'Delete this record'),"<br>\n";
	$ainc++;
    }
    if ($type eq 'cname') {
	print "CNAME: ",$doc->textfield(-name=>'cname',
					-default=>$data,
					-size=>40,
					-maxlength=>255);
	print $doc->checkbox(-name=>'del_cname',
			     -label=>'Delete this record'),"<br>\n";
    }
    if ($type eq 'mx') {
	my($pref, $mxer) = split(/\s+/,$data);
	print $doc->popup_menu(-name=>"mx_pref$mxinc",
			       -values=>[0,5,10,20,30,40,50],
			       -default=>$pref),
	    $doc->textfield(-name=>"mx_host$mxinc",
			    -default=>$mxer,
			    -size=>40,
			    -maxlength=>255);
	print $doc->checkbox(-name=>"del_mx$mxinc",
			     -label=>'Delete this record'),"<br>\n";
	$mxinc++;
    }
    if ($type eq 'hinfo') {
	my(@list) = $dns->list_hinfo;
	print "Hardware: ",$doc->popup_menu(-name=>'hinfo',
					    -values=>\@list,
					    -default=>$data);
	print $doc->checkbox(-name=>"del_hinfo",
			     -label=>'Delete this record'),"<br>\n";
    }
    if ($type eq 'txt') {
	print "Supplementary text: ",$doc->textfield(-name=>'txt'.$txtinc++,
						     -default=>$data,
						     -size=>60,
						     -maxlength=>255);
	print $doc->checkbox(-name=>"del_txt",
			     -label=>'Delete this record'),"<br>\n";
    }
}

# New data block.
print '<hr>',$doc->h3('New records');
print 'Enter new records here.  They will be added to the ones above when you';
print ' submit the form.<br>';

if ($dns->list($host,'SOA')) {
    print "NS: ", $doc->textfield(-name=>"ns$nsinc",
				  -size=>40,
				  -maxlength=>255),'<br>';
}
print "Address: ", $doc->textfield(-name=>"a$inc_ttl",
				   -size=>5,
				   -maxlength=>6);
print $doc->textfield(-name=>"a$ainc",
		      -size=>18,
		      -maxlength=>16),'<br>';
print "MX: ", $doc->popup_menu(-name=>"mx_pref$mxinc",
			       -values=>[0,5,10,20,30,40,50]);
print $doc->textfield(-name=>"mx_host$mxinc",
		      -size=>40,
		      -maxlength=>255),'<br>';

print $doc->hidden(-name=>'host',
		   -default=>$host);
print $doc->hidden(-name=>'zone',
		   -default=>$R::zone);
print $doc->submit(-name=>'Submit Changes');
print $doc->endform;

print "\n<hr>\n";
print $doc->h3($doc->center('Raw DNS data')),"\n";

print "<table>\n";
foreach $record ($dns->list($host,'all')) {
    ($type, $data) = split(/:/,$record);
    print $doc->TR($doc->th($host),
		   $doc->th($doc->strong($type)),
		   $doc->th($data)),"\n";
    $host = "";
}
print "</table>\n";
